"use strict";

exports.__esModule = true;
exports.default = useFocus;

var _useEvent = _interopRequireDefault(require("../use-event"));

var _useLayoutEffect = _interopRequireDefault(require("../__modules__/useLayoutEffect"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/**
 * useFocusEvents implementation
 */
var bubbleOptions = {
  passive: true
};
var captureOptions = {
  capture: true,
  passive: true
};

function useFocus(targetRef, config) {
  var disabled = config.disabled,
      onBlur = config.onBlur,
      onBlurCapture = config.onBlurCapture,
      onFocus = config.onFocus,
      onFocusCapture = config.onFocusCapture,
      onFocusChange = config.onFocusChange; // Event handles (these events do not bubble)

  var addBlurListener = (0, _useEvent.default)('blur', bubbleOptions);
  var addFocusListener = (0, _useEvent.default)('focus', bubbleOptions);
  var addBlurCaptureListener = (0, _useEvent.default)('blur', captureOptions);
  var addFocusCaptureListener = (0, _useEvent.default)('focus', captureOptions);
  (0, _useLayoutEffect.default)(function () {
    var target = targetRef.current;

    function blurListener(e) {
      if (onBlur != null) {
        onBlur(e);
      }

      if (onFocusChange != null) {
        onFocusChange(false, e);
      }
    }

    function focusListener(e) {
      if (onFocus != null) {
        onFocus(e);
      }

      if (onFocusChange != null) {
        onFocusChange(true, e);
      }
    }

    if (target != null) {
      addBlurListener(target, !disabled ? blurListener : null);
      addFocusListener(target, !disabled ? focusListener : null);
      addBlurCaptureListener(target, !disabled && onBlurCapture || null);
      addFocusCaptureListener(target, !disabled && onFocusCapture || null);
    }
  }, [targetRef, addBlurListener, addBlurCaptureListener, addFocusListener, addFocusCaptureListener, disabled, onBlur, onBlurCapture, onFocus, onFocusCapture, onFocusChange]);
}

module.exports = exports.default;