"use strict";

exports.__esModule = true;
exports.default = useKeyboard;

var _useEvent = _interopRequireDefault(require("../use-event"));

var _useLayoutEffect = _interopRequireDefault(require("../__modules__/useLayoutEffect"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/**
 * useKeyboardEvents implementation
 */
var captureOptions = {
  capture: true
}; // TODO: https://github.com/w3c/uievents/issues/202

function useKeyboard(targetRef, config) {
  var disabled = config.disabled,
      onKeyDown = config.onKeyDown,
      onKeyDownCapture = config.onKeyDownCapture,
      onKeyUp = config.onKeyUp,
      onKeyUpCapture = config.onKeyUpCapture; // Event handles (these events bubble)

  var addKeyDownListener = (0, _useEvent.default)('keydown');
  var addKeyDownCaptureListener = (0, _useEvent.default)('keydown', captureOptions);
  var addKeyUpListener = (0, _useEvent.default)('keyup');
  var addKeyUpCaptureListener = (0, _useEvent.default)('keyup', captureOptions);
  (0, _useLayoutEffect.default)(function () {
    var target = targetRef.current;

    var createKeyListener = function createKeyListener(listener) {
      return function keyListener(e) {
        // Ignore composition events
        if (!e.isComposing && e.keyCode !== 229) {
          if (listener != null) {
            listener(e);
          }
        }
      };
    };

    if (target != null) {
      addKeyDownListener(target, !disabled && createKeyListener(onKeyDown) || null);
      addKeyDownCaptureListener(target, !disabled && createKeyListener(onKeyDownCapture) || null);
      addKeyUpListener(target, !disabled && createKeyListener(onKeyUp) || null);
      addKeyUpCaptureListener(target, !disabled && createKeyListener(onKeyUpCapture) || null);
    }
  }, [addKeyDownListener, addKeyDownCaptureListener, addKeyUpListener, addKeyUpCaptureListener, disabled, onKeyDown, onKeyDownCapture, onKeyUp, onKeyUpCapture, targetRef]);
}

module.exports = exports.default;