"use strict";

exports.__esModule = true;
exports.default = useLayout;

var _supportsDOM = _interopRequireDefault(require("../__modules__/supportsDOM"));

var _useLayoutEffect = _interopRequireDefault(require("../__modules__/useLayoutEffect"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright (c) Nicolas Gallagher.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var HANDLE_RESIZE_KEY = '__reactHandleResize';
var didWarn = false;
var resizeObserver = null;

function getResizeObserver() {
  if ((0, _supportsDOM.default)() && typeof window.ResizeObserver !== 'undefined') {
    if (resizeObserver == null) {
      resizeObserver = new window.ResizeObserver(function (entries) {
        entries.forEach(function (entry) {
          var handleResize = entry.target[HANDLE_RESIZE_KEY];

          if (handleResize != null) {
            handleResize(entry);
          }
        });
      });
    }
  } else if (!didWarn) {
    if (process.env.NODE_ENV !== 'production' && process.env.NODE_ENV !== 'test') {
      console.warn('onLayout relies on ResizeObserver which is not supported by your browser. ' + 'Please include a polyfill, e.g., https://github.com/que-etc/resize-observer-polyfill.');
      didWarn = true;
    }
  }

  return resizeObserver;
} // TODO(#37): Support multiple uses


function useLayout(targetRef, config) {
  var observer = getResizeObserver();
  var _config$box = config.box,
      box = _config$box === void 0 ? 'border-box' : _config$box,
      onLayout = config.onLayout;
  (0, _useLayoutEffect.default)(function () {
    var target = targetRef.current;

    if (target != null && onLayout != null) {
      target[HANDLE_RESIZE_KEY] = function (entry) {
        var borderBoxSize = entry.borderBoxSize,
            contentRect = entry.contentRect; // 'contentRect' is legacy but not deprecated, and it provides the same
        // values as 'contentBoxSize' in fewer btyes

        var height = contentRect.height,
            width = contentRect.width;

        if (box === 'border-box') {
          if (borderBoxSize != null) {
            // Firefox doesn't provide a sequence of boxes, even though the spec
            // defines this value as a frozen array
            var borderSize = borderBoxSize[0] || borderBoxSize;
            height = borderSize.blockSize;
            width = borderSize.inlineSize;
          } else {
            // 'borderBoxSize' is not currently available in all engines and not
            // included in most polyfills, so we fallback to 'getBoundingClientRect'
            var borderRect = target.getBoundingClientRect();
            height = borderRect.height;
            width = borderRect.width;
          }
        }

        onLayout({
          width: width,
          height: height
        });
      };
    }
  }, [box, targetRef, onLayout]); // Observing is done in a separate effect to avoid this effect running any time
  // 'onLayout' changes.

  (0, _useLayoutEffect.default)(function () {
    var target = targetRef.current;

    if (target != null && observer != null) {
      if (typeof target[HANDLE_RESIZE_KEY] === 'function') {
        // $FlowExpectedError: the Flow type is wrong
        observer.observe(target, {
          box: box
        });
      } else {
        observer.unobserve(target);
      }
    }

    return function () {
      if (target != null && observer != null) {
        observer.unobserve(target);
      }
    };
  }, [box, targetRef, observer]);
}

module.exports = exports.default;