"use strict";

exports.__esModule = true;
exports.default = usePress;

var React = _interopRequireWildcard(require("react"));

var _checkTargetContainsSelection = _interopRequireDefault(require("../__modules__/checkTargetContainsSelection"));

var _supportsDOM = _interopRequireDefault(require("../__modules__/supportsDOM"));

var _modality = require("../__modules__/modality");

var _useKeyboard = _interopRequireDefault(require("../use-keyboard"));

var _useResponder = _interopRequireDefault(require("../use-responder"));

var _useEvent = _interopRequireDefault(require("../use-event"));

var _useLayoutEffect = _interopRequireDefault(require("../__modules__/useLayoutEffect"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var emptyObject = {};
var MOVE_THRESHOLD_PX = 10;
var DEFAULT_LONG_PRESS_DELAY_MS = 500;

function getTouchFromResponderEvent(event) {
  var _event$nativeEvent = event.nativeEvent,
      changedTouches = _event$nativeEvent.changedTouches,
      touches = _event$nativeEvent.touches;

  if (touches != null && touches.length > 0) {
    return touches[0];
  }

  if (changedTouches != null && changedTouches.length > 0) {
    return changedTouches[0];
  }

  return event.nativeEvent;
}

function isKeyActivator(key) {
  return key === 'Enter' || isKeySpacebar(key);
}

function isKeySpacebar(key) {
  return key === ' ' || key === 'Spacebar';
}

function isTargetDisabled(target) {
  return target.disabled || target.getAttribute('aria-disabled') === 'true';
}

function isTargetNativelyInteractive(target) {
  var href = target.href,
      tagName = target.tagName;
  return tagName === 'A' && href != null || tagName === 'BUTTON' || tagName === 'INPUT' || tagName === 'SELECT' || tagName === 'TEXTAREA';
}

function shouldSpacebarActivate(event) {
  var currentTarget = event.currentTarget;
  var role = currentTarget.getAttribute('role');
  var spacebarActivates = role === 'button' || role === 'checkbox' || role === 'menuitem' || role === 'menuitemcheckbox' || role === 'menuitemradio' || role === 'radio' || role === 'switch';
  return spacebarActivates;
}

function useClick(targetRef, listener) {
  var addClickListener = (0, _useEvent.default)('click');
  (0, _useLayoutEffect.default)(function () {
    var target = targetRef.current;

    if (target != null) {
      addClickListener(target, listener);
    }
  }, [addClickListener, targetRef, listener]);
}

function useContextMenu(targetRef, listener) {
  var addContextMenuListener = (0, _useEvent.default)('contextmenu');
  (0, _useLayoutEffect.default)(function () {
    var target = targetRef.current;

    if (target != null) {
      addContextMenuListener(target, listener);
    }
  }, [addContextMenuListener, targetRef, listener]);
}

function usePress(targetRef, config) {
  var _ref = config || emptyObject,
      delayLongPress = _ref.delayLongPress,
      disabled = _ref.disabled,
      onLongPress = _ref.onLongPress,
      onPress = _ref.onPress,
      onPressCancel = _ref.onPressCancel,
      onPressChange = _ref.onPressChange,
      onPressEnd = _ref.onPressEnd,
      onPressStart = _ref.onPressStart,
      touchAction = _ref.touchAction;

  var longPressDelay = delayLongPress || DEFAULT_LONG_PRESS_DELAY_MS;
  var documentRef = React.useRef((0, _supportsDOM.default)() ? document : null);
  var stateRef = React.useRef({
    active: false,
    initialX: null,
    initialY: null,
    isLongPress: false,
    longPressTimeout: null
  });
  useClick(targetRef, React.useCallback(function (e) {
    var state = stateRef.current;
    var target = e.currentTarget;

    if (isTargetDisabled(target)) {
      e.stopPropagation();
    } else if (!disabled) {
      e.stopPropagation();
      var deltaX = state.initialX != null ? state.initialX - e.pageX : 0;
      var deltaY = state.initialY != null ? state.initialY - e.pageY : 0; // The browser will select text beneath a pointer when it performs a rapid double-tap.
      // We only want to cancel press if the user moves the pointer to select text.

      var hasSelection = (0, _checkTargetContainsSelection.default)(target);
      var didSelectText = Math.hypot(deltaX, deltaY) > MOVE_THRESHOLD_PX && hasSelection; // Click triggers onPress if there is no active longpress, and no text
      // selection occured during the press. Otherwise, it is cancelled.

      if (state.isLongPress || didSelectText) {
        e.preventDefault();
      } else if (onPress != null) {
        onPress(e);
      }
    }
  }, [disabled, onPress, stateRef, targetRef]));
  useContextMenu(targetRef, React.useCallback(function (e) {
    var target = e.currentTarget;

    if (isTargetDisabled(target) || !disabled && onLongPress != null && (0, _modality.getActiveModality)() === 'touch') {
      // Context menu is cancelled for touch pointers if onLongPress is defined.
      // Context menu is cancelled if the element or hook is disabled
      e.stopPropagation();
      e.preventDefault();
    }
  }, [disabled, onLongPress]));
  /**
   * Pointers
   */

  (0, _useResponder.default)(targetRef, {
    onStartShouldSetResponder: function onStartShouldSetResponder(e) {
      var target = e.currentTarget;
      return !disabled && !isTargetDisabled(target);
    },
    onResponderGrant: function onResponderGrant(e) {
      var state = stateRef.current;
      var touch = getTouchFromResponderEvent(e);
      state.active = true;
      state.isLongPress = false;
      state.initialX = touch.pageX;
      state.initialY = touch.pageY;

      if (onPressStart != null) {
        onPressStart(e);
      }

      if (onPressChange != null) {
        onPressChange(true);
      }

      if (onLongPress != null) {
        state.longPressTimeout = setTimeout(function () {
          if (state.active === true) {
            state.isLongPress = true;
            onLongPress();
          }
        }, longPressDelay);
      }
    },
    onResponderMove: function onResponderMove(e) {
      var state = stateRef.current;
      var touch = getTouchFromResponderEvent(e);
      var deltaX = state.initialX != null ? state.initialX - touch.pageX : 0;
      var deltaY = state.initialY != null ? state.initialY - touch.pageY : 0; // If the pointer moves more than the threshold, cancel longpress

      if (onLongPress != null && Math.hypot(deltaX, deltaY) > MOVE_THRESHOLD_PX) {
        clearTimeout(state.longPressTimeout);
        state.longPressTimeout = null;
      }
    },
    onResponderRelease: function onResponderRelease(e) {
      var state = stateRef.current;

      if (onPressChange != null) {
        onPressChange(false);
      }

      if (onPressEnd != null) {
        onPressEnd(e);
      }

      clearTimeout(state.longPressTimeout);
      state.active = false;
      state.longPressTimeout = null;
    },
    onResponderTerminate: function onResponderTerminate(e) {
      var state = stateRef.current;

      if (onPressChange != null) {
        onPressChange(false);
      }

      if (onPressCancel != null) {
        onPressCancel(e);
      }

      clearTimeout(state.longPressTimeout);
      state.active = false;
      state.isLongPress = false;
      state.longPressTimeout = null;
    },
    onResponderTerminationRequest: function onResponderTerminationRequest(e) {
      if (onLongPress != null && e.nativeEvent.type === 'contextmenu' && (0, _modality.getActiveModality)() === 'touch') {
        return false;
      }

      return true;
    },
    touchAction: touchAction
  });
  /**
   * Keyboard
   */

  (0, _useKeyboard.default)(targetRef, {
    disabled: disabled,
    onKeyDown: function onKeyDown(e) {
      var key = e.key;
      var state = stateRef.current;

      if (isTargetDisabled(e.currentTarget) && !isKeySpacebar(key)) {
        e.stopPropagation();
      } else if (isKeyActivator(key)) {
        // Stop the spacebar potentially scrolling the window
        if (isKeySpacebar(key)) {
          if (shouldSpacebarActivate(e)) {
            e.preventDefault();
          } else {
            return;
          }
        }

        e.stopPropagation();

        if (state.active !== true) {
          if (onPressStart != null) {
            onPressStart(e);
          }

          if (onPressChange != null) {
            onPressChange(true);
          }

          state.active = true;
        }
      }
    }
  }); // The 'keyup' listener is attached to the document to account for situtations where
  // focus is moved during 'keydown', resulting in 'keyup' not occuring on the target.
  // In that case, we want to clear the pressed state on 'keyup' without triggering 'onPress'.
  // Use the capture phase on the off-chance that the 'keyup' target prevents propagation.

  (0, _useKeyboard.default)(documentRef, {
    disabled: disabled,
    onKeyUpCapture: function onKeyUpCapture(e) {
      var key = e.key;
      var target = targetRef.current;
      var state = stateRef.current;

      if (state.active && isKeyActivator(key)) {
        if (onPressChange != null) {
          onPressChange(false);
        }

        if (onPressEnd != null) {
          onPressEnd(e);
        }

        if (e.target === target) {
          // Only stop propagation if the key is being handled by the original target
          e.stopPropagation();

          if (onPress != null && !isTargetNativelyInteractive(e.target)) {
            onPress(e);
          }
        }

        state.active = false;
      }
    }
  });
}

module.exports = exports.default;