"use strict";

exports.__esModule = true;
exports.terminateResponder = terminateResponder;
exports.default = useResponder;

var React = _interopRequireWildcard(require("react"));

var ResponderSystem = _interopRequireWildcard(require("./ResponderSystem"));

var _useLayoutEffect = _interopRequireDefault(require("../__modules__/useLayoutEffect"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Copyright (c) Nicolas Gallagher
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

/**
 * Hook for integrating the Responder System into React
 *
 *   function SomeComponent({ onStartShouldSetResponder }) {
 *     const ref = useRef(null);
 *     useResponderEvents(ref, { onStartShouldSetResponder });
 *     return <div ref={ref} />
 *   }
 */
function terminateResponder() {
  ResponderSystem.terminateResponder();
}

function useResponder(targetRef, config) {
  // This is a separate effect so it doesn't run when the config changes.
  // On initial mount, attach global listeners if needed.
  (0, _useLayoutEffect.default)(function () {
    ResponderSystem.attachListeners();
  }, []); // Set the config as necessary

  (0, _useLayoutEffect.default)(function () {
    var node = targetRef.current;

    if (node != null) {
      ResponderSystem.setConfig(node, config);
    }
  }, [config, targetRef]);
  React.useDebugValue({
    isResponder: targetRef.current === ResponderSystem.getResponderNode()
  });
  React.useDebugValue(config);
}